@@ -5,31 +5,29 @@ import inspect
 import keyword
 import os
 import re
-import sys
 import warnings
 from collections.abc import Container, Iterable, Sized
 from pathlib import Path
 from types import MappingProxyType
 
-from yarl import URL, quote, unquote
+# do not use yarl.quote directly,
+# use `URL(path).raw_path` instead of `quote(path)`
+# Escaping of the URLs need to be consitent with the escaping done by yarl
+from yarl import URL, unquote
 
-from . import hdrs
+from . import hdrs, helpers
 from .abc import AbstractMatchInfo, AbstractRouter, AbstractView
-from .file_sender import FileSender
-from .protocol import HttpVersion11
+from .http import HttpVersion11
 from .web_exceptions import (HTTPExpectationFailed, HTTPForbidden,
                              HTTPMethodNotAllowed, HTTPNotFound)
-from .web_reqrep import Response, StreamResponse
+from .web_fileresponse import FileResponse
+from .web_response import Response, StreamResponse
 
 __all__ = ('UrlDispatcher', 'UrlMappingMatchInfo',
            'AbstractResource', 'Resource', 'PlainResource', 'DynamicResource',
            'AbstractRoute', 'ResourceRoute',
            'StaticResource', 'View')
 
-
-PY_35 = sys.version_info >= (3, 5)
-
-
 HTTP_METHOD_RE = re.compile(r"^[0-9A-Za-z!#\$%&'\*\+\-\.\^_`\|~]+$")
 
 
@@ -166,7 +164,7 @@ class UrlMappingMatchInfo(dict, AbstractMatchInfo):
     def __init__(self, match_dict, route):
         super().__init__(match_dict)
         self._route = route
-        self._apps = []
+        self._apps = ()
         self._frozen = False
 
     @property
@@ -190,12 +188,12 @@ class UrlMappingMatchInfo(dict, AbstractMatchInfo):
 
     @property
     def apps(self):
-        return tuple(self._apps)
+        return self._apps
 
     def add_app(self, app):
         if self._frozen:
             raise RuntimeError("Cannot change apps stack after .freeze() call")
-        self._apps.insert(0, app)
+        self._apps = (app,) + self._apps
 
     def freeze(self):
         self._frozen = True
@@ -229,7 +227,8 @@ def _defaultExpectHandler(request):
     expect = request.headers.get(hdrs.EXPECT)
     if request.version == HttpVersion11:
         if expect.lower() == "100-continue":
-            request.transport.write(b"HTTP/1.1 100 Continue\r\n\r\n")
+            request.writer.write(b"HTTP/1.1 100 Continue\r\n\r\n", drain=False)
+            yield from request.writer.drain()
         else:
             raise HTTPExpectationFailed(text="Unknown Expect: %s" % expect)
 
@@ -271,7 +270,8 @@ class Resource(AbstractResource):
             route_method = route.method
             allowed_methods.add(route_method)
 
-            if route_method == request.method or route_method == hdrs.METH_ANY:
+            if (route_method == request._method or
+                    route_method == hdrs.METH_ANY):
                 return UrlMappingMatchInfo(match_dict, route), allowed_methods
         else:
             return None, allowed_methods
@@ -373,7 +373,7 @@ class PrefixResource(AbstractResource):
         assert not prefix or prefix.startswith('/'), prefix
         assert prefix in ('', '/') or not prefix.endswith('/'), prefix
         super().__init__(name=name)
-        self._prefix = quote(prefix, safe='/')
+        self._prefix = URL(prefix).raw_path
 
     def add_prefix(self, prefix):
         assert prefix.startswith('/')
@@ -400,9 +400,8 @@ class StaticResource(PrefixResource):
             raise ValueError(
                 "No directory exists at '{}'".format(directory)) from error
         self._directory = directory
-        self._file_sender = FileSender(resp_factory=response_factory,
-                                       chunk_size=chunk_size)
         self._show_index = show_index
+        self._chunk_size = chunk_size
         self._follow_symlinks = follow_symlinks
         self._expect_handler = expect_handler
 
@@ -421,7 +420,7 @@ class StaticResource(PrefixResource):
         while filename.startswith('/'):
             filename = filename[1:]
         filename = '/' + filename
-        url = self._prefix + quote(filename, safe='/')
+        url = self._prefix + URL(filename).raw_path
         return URL(url)
 
     def get_info(self):
@@ -438,7 +437,7 @@ class StaticResource(PrefixResource):
     @asyncio.coroutine
     def resolve(self, request):
         path = request.rel_url.raw_path
-        method = request.method
+        method = request._method
         allowed_methods = set(self._routes)
         if not path.startswith(self._prefix):
             return None, set()
@@ -483,7 +482,7 @@ class StaticResource(PrefixResource):
             else:
                 raise HTTPForbidden()
         elif filepath.is_file():
-            ret = yield from self._file_sender.send(request, filepath)
+            ret = FileResponse(filepath, chunk_size=self._chunk_size)
         else:
             raise HTTPNotFound
 
@@ -650,15 +649,15 @@ class View(AbstractView):
 
     @asyncio.coroutine
     def __iter__(self):
-        if self.request.method not in hdrs.METH_ALL:
+        if self.request._method not in hdrs.METH_ALL:
             self._raise_allowed_methods()
-        method = getattr(self, self.request.method.lower(), None)
+        method = getattr(self, self.request._method.lower(), None)
         if method is None:
             self._raise_allowed_methods()
         resp = yield from method()
         return resp
 
-    if PY_35:
+    if helpers.PY_35:
         def __await__(self):
             return (yield from self.__iter__())
 
@@ -710,15 +709,14 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
     ROUTE_RE = re.compile(r'(\{[_a-zA-Z][^{}]*(?:\{[^{}]*\}[^{}]*)*\})')
     NAME_SPLIT_RE = re.compile(r'[.:-]')
 
-    def __init__(self, app):
+    def __init__(self):
         super().__init__()
         self._resources = []
         self._named_resources = {}
-        self._app = app
 
     @asyncio.coroutine
     def resolve(self, request):
-        method = request.method
+        method = request._method
         allowed_methods = set()
 
         for resource in self._resources:
@@ -755,13 +753,13 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
     def named_resources(self):
         return MappingProxyType(self._named_resources)
 
-    def _reg_resource(self, resource):
+    def register_resource(self, resource):
         assert isinstance(resource, AbstractResource), \
             'Instance of AbstractResource class is required, got {!r}'.format(
                 resource)
         if self.frozen:
-            raise RuntimeError("Cannot register a resource into "
-                               "frozen router.")
+            raise RuntimeError(
+                "Cannot register a resource into frozen router.")
 
         name = resource.name
 
@@ -784,8 +782,9 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
         if path and not path.startswith('/'):
             raise ValueError("path should be started with / or be empty")
         if not ('{' in path or '}' in path or self.ROUTE_RE.search(path)):
-            resource = PlainResource(quote(path, safe='/'), name=name)
-            self._reg_resource(resource)
+            url = URL(path)
+            resource = PlainResource(url.raw_path, name=name)
+            self.register_resource(resource)
             return resource
 
         pattern = ''
@@ -806,9 +805,9 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
             if '{' in part or '}' in part:
                 raise ValueError("Invalid path '{}'['{}']".format(path, part))
 
-            part = quote(part, safe='/')
-            formatter += part
-            pattern += re.escape(part)
+            path = URL(part).raw_path
+            formatter += path
+            pattern += re.escape(path)
 
         try:
             compiled = re.compile(pattern)
@@ -816,7 +815,7 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
             raise ValueError(
                 "Bad pattern '{}': {}".format(pattern, exc)) from None
         resource = DynamicResource(compiled, formatter, name=name)
-        self._reg_resource(resource)
+        self.register_resource(resource)
         return resource
 
     def add_route(self, method, path, handler,
@@ -845,7 +844,7 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
                                   response_factory=response_factory,
                                   show_index=show_index,
                                   follow_symlinks=follow_symlinks)
-        self._reg_resource(resource)
+        self.register_resource(resource)
         return resource
 
     def add_head(self, *args, **kwargs):
@@ -854,11 +853,16 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
         """
         return self.add_route(hdrs.METH_HEAD, *args, **kwargs)
 
-    def add_get(self, *args, **kwargs):
+    def add_get(self, *args, name=None, allow_head=True, **kwargs):
         """
-        Shortcut for add_route with method GET
+        Shortcut for add_route with method GET, if allow_head is true another
+        route is added allowing head requests to the same endpoint
         """
-        return self.add_route(hdrs.METH_GET, *args, **kwargs)
+        if allow_head:
+            # the head route can't have "name" set or it would conflict with
+            # the GET route below
+            self.add_route(hdrs.METH_HEAD, *args, **kwargs)
+        return self.add_route(hdrs.METH_GET, *args, name=name, **kwargs)
 
     def add_post(self, *args, **kwargs):
         """
@@ -884,19 +888,6 @@ class UrlDispatcher(AbstractRouter, collections.abc.Mapping):
         """
         return self.add_route(hdrs.METH_DELETE, *args, **kwargs)
 
-    def add_subapp(self, prefix, subapp):
-        if subapp.frozen:
-            raise RuntimeError("Cannod add frozen application")
-        if prefix.endswith('/'):
-            prefix = prefix[:-1]
-        if prefix in ('', '/'):
-            raise ValueError("Prefix cannot be empty")
-        resource = PrefixedSubAppResource(prefix, subapp)
-        self._reg_resource(resource)
-        self._app._reg_subapp_signals(subapp)
-        subapp.freeze()
-        return resource
-
     def freeze(self):
         super().freeze()
         for resource in self._resources:
