@@ -13,24 +13,29 @@ from aiohttp import helpers
 from aiohttp.client_reqrep import ClientResponse
 
 
-def test_del(loop):
+def test_del():
+    loop = mock.Mock()
     response = ClientResponse('get', URL('http://del-cl-resp.org'))
     response._post_init(loop)
+    loop.get_debug = mock.Mock()
+    loop.get_debug.return_value = True
 
     connection = mock.Mock()
-    response._setup_connection(connection)
+    response._closed = False
+    response._connection = connection
     loop.set_exception_handler(lambda loop, ctx: None)
 
     with pytest.warns(ResourceWarning):
         del response
         gc.collect()
 
-    connection.close.assert_called_with()
+    connection.release.assert_called_with()
 
 
 def test_close(loop):
     response = ClientResponse('get', URL('http://def-cl-resp.org'))
     response._post_init(loop)
+    response._closed = False
     response._connection = mock.Mock()
     response.close()
     assert response.connection is None
@@ -76,6 +81,12 @@ def test_repr_non_ascii_reason():
         in repr(response)
 
 
+def test_url_obj_deprecated():
+    response = ClientResponse('get', URL('http://fake-host.org/'))
+    with pytest.warns(DeprecationWarning):
+        response.url_obj
+
+
 @asyncio.coroutine
 def test_read_and_release_connection(loop):
     response = ClientResponse('get', URL('http://def-cl-resp.org'))
@@ -115,10 +126,52 @@ def test_release(loop):
     content = response.content = mock.Mock()
     content.readany.return_value = fut
 
-    yield from response.release()
+    response.release()
     assert response._connection is None
 
 
+@asyncio.coroutine
+def test_release_on_del(loop):
+    connection = mock.Mock()
+    connection.protocol.upgraded = False
+
+    def run(conn):
+        response = ClientResponse('get', URL('http://def-cl-resp.org'))
+        response._post_init(loop)
+        response._closed = False
+        response._connection = conn
+
+    run(connection)
+
+    assert connection.release.called
+
+
+@asyncio.coroutine
+def test_response_eof(loop):
+    response = ClientResponse('get', URL('http://def-cl-resp.org'))
+    response._post_init(loop)
+    response._closed = False
+    conn = response._connection = mock.Mock()
+    conn.protocol.upgraded = False
+
+    response._response_eof()
+    assert conn.release.called
+    assert response._connection is None
+
+
+@asyncio.coroutine
+def test_response_eof_upgraded(loop):
+    response = ClientResponse('get', URL('http://def-cl-resp.org'))
+    response._post_init(loop)
+
+    conn = response._connection = mock.Mock()
+    conn.protocol.upgraded = True
+
+    response._response_eof()
+    assert not conn.release.called
+    assert response._connection is conn
+
+
 @asyncio.coroutine
 def test_text(loop):
     response = ClientResponse('get', URL('http://def-cl-resp.org'))
@@ -267,13 +320,11 @@ def test_json_no_content(loop):
         'Content-Type': 'data/octet-stream'}
     response._content = b''
 
-    with mock.patch('aiohttp.client_reqrep.client_logger') as m_log:
-        res = yield from response.json()
+    with pytest.raises(aiohttp.ClientResponseError):
+        yield from response.json()
 
+    res = yield from response.json(content_type=None)
     assert res is None
-    m_log.warning.assert_called_with(
-        'Attempt to decode JSON with unexpected mimetype: %s',
-        'data/octet-stream')
 
 
 @asyncio.coroutine
@@ -298,12 +349,13 @@ def test_json_override_encoding(loop):
     assert not response._get_encoding.called
 
 
+@pytest.mark.xfail
 def test_override_flow_control(loop):
     class MyResponse(ClientResponse):
         flow_control_class = aiohttp.StreamReader
     response = MyResponse('get', URL('http://my-cl-resp.org'))
     response._post_init(loop)
-    response._setup_connection(mock.Mock())
+    response._connection = mock.Mock()
     assert isinstance(response.content, aiohttp.StreamReader)
     response.close()
 
@@ -329,7 +381,7 @@ def test_raise_for_status_4xx():
     response = ClientResponse('get', URL('http://def-cl-resp.org'))
     response.status = 409
     response.reason = 'CONFLICT'
-    with pytest.raises(aiohttp.HttpProcessingError) as cm:
+    with pytest.raises(aiohttp.ClientResponseError) as cm:
         response.raise_for_status()
     assert str(cm.value.code) == '409'
     assert str(cm.value.message) == "CONFLICT"
@@ -337,8 +389,7 @@ def test_raise_for_status_4xx():
 
 def test_resp_host():
     response = ClientResponse('get', URL('http://del-cl-resp.org'))
-    with pytest.warns(DeprecationWarning):
-        assert 'del-cl-resp.org' == response.host
+    assert 'del-cl-resp.org' == response.host
 
 
 def test_content_type():
