@@ -6,6 +6,14 @@ import aiohttp
 from aiohttp import hdrs, helpers, web
 
 
+@pytest.fixture
+def ceil(mocker):
+    def ceil(val):
+        return val
+
+    mocker.patch('aiohttp.helpers.ceil').side_effect = ceil
+
+
 @asyncio.coroutine
 def test_send_recv_text(loop, test_client):
 
@@ -19,16 +27,20 @@ def test_send_recv_text(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
     resp.send_str('ask')
 
+    assert resp.get_extra_info('socket') is not None
+
     data = yield from resp.receive_str()
     assert data == 'ask/answer'
     yield from resp.close()
 
+    assert resp.get_extra_info('socket') is None
+
 
 @asyncio.coroutine
 def test_send_recv_bytes_bad_type(loop, test_client):
@@ -43,7 +55,7 @@ def test_send_recv_bytes_bad_type(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -67,7 +79,7 @@ def test_send_recv_bytes(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -93,7 +105,7 @@ def test_send_recv_text_bad_type(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -119,7 +131,7 @@ def test_send_recv_json(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -150,7 +162,7 @@ def test_ping_pong(loop, test_client):
             closed.set_result(1)
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -188,7 +200,7 @@ def test_ping_pong_manual(loop, test_client):
             closed.set_result(1)
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', autoping=False)
@@ -226,7 +238,7 @@ def test_close(loop, test_client):
         yield from ws.receive()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -261,7 +273,7 @@ def test_concurrent_close(loop, test_client):
         assert msg.type == aiohttp.WSMsgType.CLOSE
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     ws = client_ws = yield from client.ws_connect('/')
@@ -293,7 +305,7 @@ def test_close_from_server(loop, test_client):
             closed.set_result(1)
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -329,7 +341,7 @@ def test_close_manual(loop, test_client):
             closed.set_result(1)
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', autoclose=False)
@@ -358,9 +370,10 @@ def test_close_timeout(loop, test_client):
         yield from ws.prepare(request)
         yield from ws.receive_bytes()
         ws.send_str('test')
-        yield from asyncio.sleep(10, loop=loop)
+        yield from asyncio.sleep(1, loop=loop)
+        return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', timeout=0.2, autoclose=False)
@@ -387,7 +400,7 @@ def test_close_cancel(loop, test_client):
         ws.send_str('test')
         yield from asyncio.sleep(10, loop=loop)
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', autoclose=False)
@@ -413,12 +426,11 @@ def test_override_default_headers(loop, test_client):
         assert request.headers[hdrs.SEC_WEBSOCKET_VERSION] == '8'
         ws = web.WebSocketResponse()
         yield from ws.prepare(request)
-
-        ws.send_str('answer')
+        yield from ws.send_str('answer')
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     headers = {hdrs.SEC_WEBSOCKET_VERSION: '8'}
     client = yield from test_client(app)
@@ -441,7 +453,7 @@ def test_additional_headers(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', headers={'x-hdr': 'xtra'})
@@ -463,7 +475,7 @@ def test_recv_protocol_error(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -492,7 +504,7 @@ def test_recv_timeout(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
@@ -516,12 +528,11 @@ def test_receive_timeout(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
 
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/', receive_timeout=0.1)
-    resp._time_service._interval = 0.05
 
     with pytest.raises(asyncio.TimeoutError):
         yield from resp.receive(0.05)
@@ -540,12 +551,11 @@ def test_custom_receive_timeout(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
 
     client = yield from test_client(app)
     resp = yield from client.ws_connect('/')
-    resp._time_service._interval = 0.05
 
     with pytest.raises(asyncio.TimeoutError):
         yield from resp.receive(0.05)
@@ -554,7 +564,7 @@ def test_custom_receive_timeout(loop, test_client):
 
 
 @asyncio.coroutine
-def test_heartbeat(loop, test_client):
+def test_heartbeat(loop, test_client, ceil):
     ping_received = False
 
     @asyncio.coroutine
@@ -568,7 +578,7 @@ def test_heartbeat(loop, test_client):
         yield from ws.close()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
 
     client = yield from test_client(app)
@@ -581,7 +591,7 @@ def test_heartbeat(loop, test_client):
 
 
 @asyncio.coroutine
-def test_heartbeat_no_pong(loop, test_client):
+def test_heartbeat_no_pong(loop, test_client, ceil):
     ping_received = False
 
     @asyncio.coroutine
@@ -595,7 +605,7 @@ def test_heartbeat_no_pong(loop, test_client):
         yield from ws.receive()
         return ws
 
-    app = web.Application(loop=loop)
+    app = web.Application()
     app.router.add_route('GET', '/', handler)
 
     client = yield from test_client(app)
