@@ -4,6 +4,7 @@ from unittest import mock
 import pytest
 
 from aiohttp import helpers, log, web
+from aiohttp.abc import AbstractRouter
 
 
 def test_app_ctor(loop):
@@ -12,46 +13,53 @@ def test_app_ctor(loop):
     assert app.logger is log.web_logger
 
 
-def test_app_call(loop):
-    app = web.Application(loop=loop)
+def test_app_call():
+    app = web.Application()
     assert app is app()
 
 
-def test_app_copy(loop):
-    app = web.Application(loop=loop)
-    with pytest.raises(NotImplementedError):
-        app.copy()
+def test_app_default_loop():
+    app = web.Application()
+    assert app.loop is None
 
 
-def test_app_default_loop(loop):
+def test_set_loop(loop):
+    app = web.Application()
+    app._set_loop(loop)
+    assert app.loop is loop
+
+
+def test_set_loop_default_loop(loop):
     asyncio.set_event_loop(loop)
     app = web.Application()
-    assert loop is app.loop
+    app._set_loop(None)
+    assert app.loop is loop
+
+
+def test_set_loop_with_different_loops(loop):
+    app = web.Application()
+    app._set_loop(loop)
+    assert app.loop is loop
+
+    with pytest.raises(RuntimeError):
+        app._set_loop(loop=object())
 
 
 @pytest.mark.parametrize('debug', [True, False])
 def test_app_make_handler_debug_exc(loop, mocker, debug):
-    app = web.Application(loop=loop, debug=debug)
+    app = web.Application(debug=debug)
+    srv = mocker.patch('aiohttp.web.Server')
 
-    mocker.spy(app, '_handler_factory')
-
-    app.make_handler()
-    with pytest.warns(DeprecationWarning) as exc:
-        app.make_handler(debug=debug)
-
-    assert 'parameter is deprecated' in exc[0].message.args[0]
-    assert app._handler_factory.call_count == 2
-    app._handler_factory.assert_called_with(app, app.router, loop=loop,
-                                            debug=debug)
-
-    with pytest.raises(ValueError) as exc:
-        app.make_handler(debug=not debug)
-    assert 'The value of `debug` parameter conflicts with the' in str(exc)
+    app.make_handler(loop=loop)
+    srv.assert_called_with(app._handle,
+                           request_factory=app._make_request,
+                           loop=loop,
+                           debug=debug)
 
 
 @asyncio.coroutine
-def test_app_register_on_finish(loop):
-    app = web.Application(loop=loop)
+def test_app_register_on_finish():
+    app = web.Application()
     cb1 = mock.Mock()
     cb2 = mock.Mock()
     app.on_cleanup.append(cb1)
@@ -63,8 +71,7 @@ def test_app_register_on_finish(loop):
 
 @asyncio.coroutine
 def test_app_register_coro(loop):
-    app = web.Application(loop=loop)
-
+    app = web.Application()
     fut = helpers.create_future(loop)
 
     @asyncio.coroutine
@@ -78,36 +85,22 @@ def test_app_register_coro(loop):
     assert 123 == fut.result()
 
 
-@asyncio.coroutine
-def test_app_register_and_finish_are_deprecated(loop):
-    app = web.Application(loop=loop)
-    cb1 = mock.Mock()
-    cb2 = mock.Mock()
-    with pytest.warns(DeprecationWarning):
-        app.register_on_finish(cb1, 1, b=2)
-    with pytest.warns(DeprecationWarning):
-        app.register_on_finish(cb2, 2, c=3)
-    with pytest.warns(DeprecationWarning):
-        yield from app.finish()
-    cb1.assert_called_once_with(app, 1, b=2)
-    cb2.assert_called_once_with(app, 2, c=3)
-
-
-def test_non_default_router(loop):
-    router = web.UrlDispatcher()
-    app = web.Application(loop=loop, router=router)
+def test_non_default_router():
+    router = mock.Mock(spec=AbstractRouter)
+    app = web.Application(router=router)
     assert router is app.router
 
-    def test_logging(self):
-        logger = mock.Mock()
-        app = web.Application(loop=self.loop)
-        app.logger = logger
-        self.assertIs(app.logger, logger)
+
+def test_logging():
+    logger = mock.Mock()
+    app = web.Application()
+    app.logger = logger
+    assert app.logger is logger
 
 
 @asyncio.coroutine
-def test_on_shutdown(loop):
-    app = web.Application(loop=loop)
+def test_on_shutdown():
+    app = web.Application()
     called = False
 
     @asyncio.coroutine
@@ -124,7 +117,8 @@ def test_on_shutdown(loop):
 
 @asyncio.coroutine
 def test_on_startup(loop):
-    app = web.Application(loop=loop)
+    app = web.Application()
+    app._set_loop(loop)
 
     blocking_called = False
     long_running1_called = False
@@ -165,3 +159,24 @@ def test_on_startup(loop):
     assert long_running1_called
     assert long_running2_called
     assert all_long_running_called
+
+
+def test_app_delitem():
+    app = web.Application()
+    app['key'] = 'value'
+    assert len(app) == 1
+    del app['key']
+    assert len(app) == 0
+
+
+def test_secure_proxy_ssl_header_default():
+    app = web.Application()
+    assert app._secure_proxy_ssl_header is None
+
+
+@asyncio.coroutine
+def test_secure_proxy_ssl_header_non_default(loop):
+    app = web.Application()
+    hdr = ('X-Forwarded-Proto', 'https')
+    app.make_handler(secure_proxy_ssl_header=hdr, loop=loop)
+    assert app._secure_proxy_ssl_header is hdr
